import { AxiosInstance, AxiosPromise, AxiosRequestConfig } from 'axios';
export declare type ValidURLParts = string | number;
export declare const defaultAxiosInstance: AxiosInstance;
export declare type HttpParams = any;
export declare class Resource<T> {
    private endpoint;
    /**
     * Axios instance.
     * @type {AxiosInstance}
     */
    static client: AxiosInstance;
    /**
     * Autoappends slash to all URLs.
     * @type {boolean}
     */
    static appendSlash: boolean;
    /**
     * Instance values.
     */
    private client;
    private appendSlash;
    /**
     * Constructor.
     * @param {string} endpoint
     */
    constructor(endpoint: string);
    /**
     * Override default client for this instance.
     *
     * @param {AxiosInstance} client
     * @returns {Resource}
     */
    withClient(client: AxiosInstance): Resource<T>;
    /**
     * Override default appendSlash for this instance.
     *
     * @param {boolean} appendSlash
     * @returns {Resource}
     */
    withAppendSlash(appendSlash: boolean): Resource<T>;
    /**
     * Get a single object.
     * Makes HTTP GET request.
     *
     * @param {number} id
     * @param {AxiosRequestConfig} config
     * @returns {AxiosPromise<T>}
     */
    retrieve(id: number, config?: AxiosRequestConfig): AxiosPromise<T>;
    /**
     * Get an array of objects.
     * Makes HTTP GET request.
     *
     * @param {AxiosRequestConfig} config
     * @returns {AxiosPromise<T>}
     */
    list(config?: AxiosRequestConfig): AxiosPromise<T>;
    /**
     * Create an object.
     * Makes HTTP POST request.
     *
     * @param {HttpParams} data
     * @param {AxiosRequestConfig} config
     * @returns {AxiosPromise<T>}
     */
    create(data: HttpParams, config?: AxiosRequestConfig): AxiosPromise<T>;
    /**
     * Replace (update) an object.
     * Makes HTTP PUT request.
     *
     * @param {number} id
     * @param {HttpParams} data
     * @param {AxiosRequestConfig} config
     * @returns {AxiosPromise<T>}
     */
    update(id: number, data?: HttpParams, config?: AxiosRequestConfig): AxiosPromise<T>;
    /**
     * Update an object.
     * Makes HTTP PATCH request.
     *
     * @param {number} id
     * @param {HttpParams} data
     * @param {AxiosRequestConfig} config
     * @returns {AxiosPromise<T>}
     */
    partialUpdate(id: number, data?: HttpParams, config?: AxiosRequestConfig): AxiosPromise<T>;
    /**
     * Delete an object.
     * Makes HTTP DELETE request.
     *
     * @param {number} id
     * @param {AxiosRequestConfig} config
     * @returns {AxiosPromise}
     */
    destroy(id: number, config?: AxiosRequestConfig): AxiosPromise;
    /**
     * Creates or replaces an object.
     * If id, then updates otherwise creates.
     * Makes HTTP POST | PUT request.
     *
     * @param {HttpParams} data
     * @param {number} id
     * @param {AxiosRequestConfig} config
     * @returns {AxiosPromise<T>}
     */
    createOrUpdate(data?: HttpParams, id?: number, config?: AxiosRequestConfig): AxiosPromise<T>;
    /**
     * Creates or partially updates an object.
     * If id, then updates otherwise creates.
     * Makes HTTP POST | PATCH request.
     *
     * @param {HttpParams} data
     * @param {number} id
     * @param {AxiosRequestConfig} config
     * @returns {AxiosPromise<T>}
     */
    createOrPartialUpdate(data?: HttpParams, id?: number, config?: AxiosRequestConfig): AxiosPromise<T>;
    get(url: string, config?: AxiosRequestConfig): AxiosPromise<T>;
    post(url: string, data?: HttpParams, config?: AxiosRequestConfig): AxiosPromise<T>;
    put(url: string, data?: HttpParams, config?: AxiosRequestConfig): AxiosPromise<T>;
    patch(url: string, data?: HttpParams, config?: AxiosRequestConfig): AxiosPromise<T>;
    delete(url: string, config?: AxiosRequestConfig): AxiosPromise;
    /**
     * Build an URL from parts.
     * Pass an array of arguments to build a final URL.
     * Respects appendSlash option.
     *
     * Example (appendSlash = true):
     *    Resource.url('user', 'update', 100, 'force')
     *
     * Out: /api/user/update/100/force/
     *
     * @param {ValidURLParts} args
     * @returns {string}
     */
    protected url(...args: ValidURLParts[]): string;
}
export default Resource;
