import axios, { AxiosInstance, AxiosPromise, AxiosRequestConfig } from 'axios';

export type ValidURLParts = string | number;

export const defaultAxiosInstance = axios.create();

export type HttpParams = any;

export class Resource<T> {
    /**
     * Axios instance.
     * @type {AxiosInstance}
     */
    public static client: AxiosInstance = defaultAxiosInstance;

    /**
     * Autoappends slash to all URLs.
     * @type {boolean}
     */
    public static appendSlash: boolean = true;

    /**
     * Instance values.
     */
    private client: AxiosInstance = Resource.client;
    private appendSlash: boolean = Resource.appendSlash;

    /**
     * Constructor.
     * @param {string} endpoint
     */
    constructor(private endpoint: string) {
        if (endpoint[0] === '/') {
            this.endpoint = endpoint.substr(1);
        }

        this.client.interceptors.request.use((config) => {
            if (this.appendSlash && !config.url.endsWith('/')) {
                config.url += '/';
            }
            return config;
        });
    }

    // per instance configuration methods

    /**
     * Override default client for this instance.
     *
     * @param {AxiosInstance} client
     * @returns {Resource}
     */
    public withClient(client: AxiosInstance): Resource<T> {
        this.client = client;
        return this;
    }

    /**
     * Override default appendSlash for this instance.
     *
     * @param {boolean} appendSlash
     * @returns {Resource}
     */
    public withAppendSlash(appendSlash: boolean): Resource<T> {
        this.appendSlash = appendSlash;
        return this;
    }

    // semantic helpers

    /**
     * Get a single object.
     * Makes HTTP GET request.
     *
     * @param {number} id
     * @param {AxiosRequestConfig} config
     * @returns {AxiosPromise<T>}
     */
    public retrieve(id: number, config?: AxiosRequestConfig): AxiosPromise<T> {
        return this.get(this.url(id), config);
    }

    /**
     * Get an array of objects.
     * Makes HTTP GET request.
     *
     * @param {AxiosRequestConfig} config
     * @returns {AxiosPromise<T>}
     */
    public list(config?: AxiosRequestConfig): AxiosPromise<T> {
        return this.get(this.url(), config);
    }

    /**
     * Create an object.
     * Makes HTTP POST request.
     *
     * @param {HttpParams} data
     * @param {AxiosRequestConfig} config
     * @returns {AxiosPromise<T>}
     */
    public create(data: HttpParams, config?: AxiosRequestConfig): AxiosPromise<T> {
        return this.post(this.url(), data, config);
    }

    /**
     * Replace (update) an object.
     * Makes HTTP PUT request.
     *
     * @param {number} id
     * @param {HttpParams} data
     * @param {AxiosRequestConfig} config
     * @returns {AxiosPromise<T>}
     */
    public update(id: number, data?: HttpParams, config?: AxiosRequestConfig): AxiosPromise<T> {
        return this.put(this.url(id), data, config);
    }

    /**
     * Update an object.
     * Makes HTTP PATCH request.
     *
     * @param {number} id
     * @param {HttpParams} data
     * @param {AxiosRequestConfig} config
     * @returns {AxiosPromise<T>}
     */
    public partialUpdate(id: number, data?: HttpParams, config?: AxiosRequestConfig): AxiosPromise<T> {
        return this.patch(this.url(id), data, config);
    }

    /**
     * Delete an object.
     * Makes HTTP DELETE request.
     *
     * @param {number} id
     * @param {AxiosRequestConfig} config
     * @returns {AxiosPromise}
     */
    public destroy(id: number, config?: AxiosRequestConfig): AxiosPromise {
        return this.delete(this.url(id), config);
    }

    /**
     * Creates or replaces an object.
     * If id, then updates otherwise creates.
     * Makes HTTP POST | PUT request.
     *
     * @param {HttpParams} data
     * @param {number} id
     * @param {AxiosRequestConfig} config
     * @returns {AxiosPromise<T>}
     */
    public createOrUpdate(data?: HttpParams, id?: number, config?: AxiosRequestConfig): AxiosPromise<T> {
        if (id) {
            return this.update(id, data, config);
        }
        return this.create(data, config);
    }

    /**
     * Creates or partially updates an object.
     * If id, then updates otherwise creates.
     * Makes HTTP POST | PATCH request.
     *
     * @param {HttpParams} data
     * @param {number} id
     * @param {AxiosRequestConfig} config
     * @returns {AxiosPromise<T>}
     */
    public createOrPartialUpdate(data?: HttpParams, id?: number, config?: AxiosRequestConfig): AxiosPromise<T> {
        if (id) {
            return this.partialUpdate(id, data, config);
        }
        return this.create(data, config);
    }

    // generic methods here

    public get(url: string, config?: AxiosRequestConfig): AxiosPromise<T> {
        return this.client.get(url, config);
    }

    public post(url: string, data?: HttpParams, config?: AxiosRequestConfig): AxiosPromise<T> {
        return this.client.post(url, data, config);
    }

    public put(url: string, data?: HttpParams, config?: AxiosRequestConfig): AxiosPromise<T> {
        return this.client.put(url, data, config);
    }

    public patch(url: string, data?: HttpParams, config?: AxiosRequestConfig): AxiosPromise<T> {
        return this.client.patch(url, data, config);
    }

    public delete(url: string, config?: AxiosRequestConfig): AxiosPromise {
        return this.client.delete(url, config);
    }

    /**
     * Build an URL from parts.
     * Pass an array of arguments to build a final URL.
     * Respects appendSlash option.
     *
     * Example (appendSlash = true):
     *    Resource.url('user', 'update', 100, 'force')
     *
     * Out: /api/user/update/100/force/
     *
     * @param {ValidURLParts} args
     * @returns {string}
     */
    protected url(...args: ValidURLParts[]): string {
        return [this.endpoint, ...args].join('/');
    }

}

export default Resource;
